/*

 A little applet demonstrating Doppler shift
 
 Douglas Repetto, Oct 2011
 
 NOTE: the pitches played are only there to 
 help you keep track of when the wavefronts
 arrive at each point. They do not represent
 the frequency of the source! The actual
 audio frequency of this source would be
 very low, well below audio range. The thing
 to pay attention to is not the pitch of the 
 bleeps themselves, but the timing of their
 arrival at each point.
 
 */

import com.jsyn.engine.SynthesisEngine;
import com.jsyn.unitgen.LineOut;
import com.jsyn.unitgen.SineOscillator;

SynthesisEngine synth;
SineOscillator leftOsc;
SineOscillator rightOsc;
LineOut lineOut;


int w = 1000;
int h = 500;
int midW = w/2;
int midH = h/2;

float sourceX = midW;
float sourceY = midH;

float sourceShiftX = 0.0;
float sourceShiftY = 0.0;


//background color
color bgC = color(0, 0, 0);
//sound source color
color sC = color(255, 255, 255);
//wave color
color wC = color(0, 127, 127);
//text color
color tC = color(255, 0, 0);

int step = 0;
//steps per cycle
double sPC = 40;


//wave front number
int wFN = 0;

WaveFront waveFronts[] = new WaveFront[100];


void setup()
{
  size(w, h);
  //frameRate(20);
  smooth();

  //Create a context for the synthesizer.
  synth = new SynthesisEngine();
  //Add a tone generator.
  synth.add( leftOsc = new SineOscillator() );
  synth.add( rightOsc = new SineOscillator() );
  //Add an output mixer.
  synth.add( lineOut = new LineOut() );
  //Connect the oscillator to the output.
  leftOsc.output.connect( 0, lineOut.input, 0 );
  rightOsc.output.connect( 0, lineOut.input, 1 );

  leftOsc.frequency.set(0.0);
  rightOsc.frequency.set(0.0);

  // Start synthesizer using default stereo output at 44100 Hz.
  synth.start();
  //We only need to start the LineOut. It will pull data from the oscillator.
  lineOut.start();

}


void draw()
{
  if (step % sPC == 0)
  {
    //create a wave front
    waveFronts[wFN] = new WaveFront(int(sourceX), int(sourceY), 10, 1);
    wFN++;

    if (wFN == 100)
      wFN = 0;
  }

  background(bgC);

  fill(tC);
  text("click mouse to move source", 10, h - 20);

  stroke(sC);
  fill(sC);
  ellipse(int(sourceX), int(sourceY), 10, 10);
  ellipse(0, midH, 10, 10);
  ellipse(w - 1, midH, 10, 10);

  noFill();
  stroke(wC);

  for (int i = 0; i < waveFronts.length; i++)
  {
    if (waveFronts[i] != null)
    {
      WaveFront wF = waveFronts[i];

      int x = int(wF.wFX);
      int y = int(wF.wFY);
      int d = int(wF.wFD);

      ellipse(x, y, d, d);

      wF.step();
    }
  }

  //check to see if it's time to bleep!
  color c = get(w - 1, midH);
  if (c != sC)
  {
    // println("r: " + red(c) + " g: " + green(c) + " b: " + blue(c));
    rightOsc.frequency.set(300.0);
  }
  else
    rightOsc.frequency.set(0.0);

  //check to see if it's time to bleep!
  c = get(0, midH);
  if (c != sC)
  {
    // println("r: " + red(c) + " g: " + green(c) + " b: " + blue(c));
    leftOsc.frequency.set(400.0);
  }
  else
    leftOsc.frequency.set(0.0);

  if (step % 3 == 0)
  {
    sourceX += sourceShiftX;

    if (sourceX >= w || sourceX <= 0.0)
      sourceShiftX *= -1.0;

    sourceY += sourceShiftY;

    if (sourceY >= h || sourceY <= 0.0)
      sourceShiftY *= -1.0;
  }
  step++;
}

void mousePressed()
{
  int x = mouseX;
  int y = mouseY;


  float xDiff = mouseX - sourceX;
  float yDiff = mouseY - sourceY;

  float distance = sqrt((xDiff * xDiff) + (yDiff * yDiff));

  println("distance: " + distance);

  sourceShiftX = xDiff/distance;
  sourceShiftY = yDiff/distance;
  println("new x: " + sourceShiftX + " new y: " + sourceShiftY);
}

